@@ -1,25 +1,13 @@
 "rrp.dist" <- function (X, treated = NULL, msplit = 10, Rep = 250, cut.in = 15, 
-    check.bal = FALSE, plot = FALSE) 
+    check.bal = FALSE, plot = FALSE, asdist=FALSE) 
 {
     n <- dim(X)[1]
     n.var <- dim(X)[2]
-    RRP <- rep(as.integer(Rep), n * (n - 1)/2)
-    class(RRP) <- "dist"
-    attr(RRP, "Size") <- n
-    attr(RRP, "Diag") <- FALSE
-    attr(RRP, "Upper") <- FALSE
-    attr(RRP, "method") <- "RRP"
-    attr(RRP, "minsplit") <- msplit
-    attr(RRP, "replications") <- Rep
-    attr(RRP, "cov.cut") <- cut.in
-    attr(RRP, "balanced") <- check.bal
-    if (!is.null(treated)) 
-        attr(RRP, "treated") <- as.logical(treated)
-    attr(RRP, "call") <- match.call()
 	
+	RRP <- newXPtr(n, Rep);
+	t.att <- NULL
 	Y <- X
-    W <- X
-    for (i in 1:dim(X)[2]) W[, i] <- as.numeric(X[, i])
+
     if (cut.in > 0) {
         for (i in 1:n.var) {
             if (is.numeric(X[, i]) | is.integer(X[, i])) {
@@ -31,7 +19,9 @@
             }
         }
     }
+    for (i in 1:dim(X)[2]) X[, i] <- as.numeric(X[, i])
     x <- cbind(z = numeric(n), Y)
+	rm(Y)
     for (K in 1:Rep) {
     	if (plot) {
             if (is.null(treated)) 
@@ -39,7 +29,7 @@
             else plot(X[, 1:2], col = treated + 2, pch = ifelse(treated, 
                 20, 17), cex = 1)
         }
-        x[, 1] <- runif(n)
+        x$z <- runif(n)
         mod <- rpart(z ~ ., data = x, method = "anova", minsplit = msplit, 
             xval = 0, cp = 0)
         idx <- which(mod$frame$var == "<leaf>")
@@ -58,12 +48,12 @@
             residual <- numeric(0)
                 if (length(idx.t[[g]]) > 0 & length(idx.c[[g]]) > 
                   0) {
-                  mins <- apply(W[idx.t[[g]], ], 2, function(x) min(x, 
+                  mins <- apply(X[idx.t[[g]], ], 2, function(x) min(x, 
                     na.rm = TRUE))
-                  maxs <- apply(W[idx.t[[g]], ], 2, function(x) max(x, 
+                  maxs <- apply(X[idx.t[[g]], ], 2, function(x) max(x, 
                     na.rm = TRUE))
                   test <- sapply(idx.c[[g]], function(x) prod(mins <= 
-                    W[x, ]) * prod(maxs >= W[x, ]))
+                    X[x, ]) * prod(maxs >= X[x, ]))
                   test <- as.logical(test)
                   similar <- c(idx.t[[g]], (idx.c[[g]])[test])
                   residual <- (idx.c[[g]])[!test]
@@ -71,16 +61,16 @@
              new.g[[2*g-1]] <<- similar
              new.g[[2*g]] <<- residual
             if (plot) {
-                minX <- min(W[similar, 1], na.rm = TRUE)
-                maxX <- max(W[similar, 1], na.rm = TRUE)
-				minY <- min(W[similar, 2], na.rm = TRUE)
-                maxY <- max(W[similar, 2], na.rm = TRUE)
+                minX <- min(X[similar, 1], na.rm = TRUE)
+                maxX <- max(X[similar, 1], na.rm = TRUE)
+				minY <- min(X[similar, 2], na.rm = TRUE)
+                maxY <- max(X[similar, 2], na.rm = TRUE)
                 rect(minX, minY, maxX, maxY, lty = 3)
                 if (length(residual) > 0) {
-                  minX <- min(W[residual, 1], na.rm = TRUE)
-                  maxX <- max(W[residual, 1], na.rm = TRUE)
-                  minY <- min(W[residual, 2], na.rm = TRUE)
-                  maxY <- max(W[residual, 2], na.rm = TRUE)
+                  minX <- min(X[residual, 1], na.rm = TRUE)
+                  maxX <- max(X[residual, 1], na.rm = TRUE)
+                  minY <- min(X[residual, 2], na.rm = TRUE)
+                  maxY <- max(X[residual, 2], na.rm = TRUE)
                   rect(minX, minY, maxX, maxY, lty = 3)
                 }
             }
@@ -91,7 +81,8 @@
 		 sapply(1:n.leaves, check.for.bal)
 		 group <- new.g
 		 }
-        RRP <- addDist(RRP, group, -1)
+        addXPtr(RRP, group, -1.0)
+		rm(mod)
         if (K%/%100 == K/100) {
             cat("0\n")
         }
@@ -101,7 +92,27 @@
             else cat(".")
         }
     }
-    RRP <- RRP/Rep
+
+    tmp <- mulXPtr(RRP, list(1:n), 1.0/Rep);
+    t.att <- NULL
+	if(!is.null(treated))
+	 t.att <-  as.logical(treated)
+	 RRPcl <- class(RRP) # XPtrToDist apparently removes class info
+
+	if(asdist)
+	 tmp <- XPtrToDist(RRP)
+
+	attributes(tmp) <- list(Size = n, Diag = FALSE, Upper = FALSE, 
+	method = "RRP", minsplit = msplit, replications = Rep, 
+	cov.cut = cut.in, balanced = check.bal, treated = t.att,
+	call = match.call()) 
+
+	if(asdist)
+	 class(tmp) <- "dist"
+	else
+	 class(tmp) <- RRPcl  
+
+     return(invisible(tmp))
 }
 
 
